"""Add internal flag field in team table

Revision ID: 045a9142794e
Revises: 9b91a1a350a0
Create Date: 2023-04-18 23:28:37.358017

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "045a9142794e"
down_revision = "9b91a1a350a0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("team", schema=None) as batch_op:
        batch_op.add_column(sa.Column("internal_flag_url", sa.String(length=300), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("team", schema=None) as batch_op:
        batch_op.drop_column("internal_flag_url")

    # ### end Alembic commands ###
